//------------------------------------------------------------------------------
// Guitar Practice
//------------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		The guitar practice classes.
//
//	Description:   	This unit contains the classes used by
//                 the guitar practice system
//
// Created:		FeMann
// Date:			30.08.2009
//
// (c) Copyright by FeMann.
// All rights reserved.
//
//

unit CGPClasses;

interface

uses
	Windows, Forms, SysUtils, Classes, ExtCtrls, Contnrs, BASS, BASS_FX, BASSENC,
   IniFiles;

type
	// The play state
   //
   CGPPlayState = (
   	psUnknown,
   	psStopped,
       psPlaying,
       psPaused
       ) ;

	// Forward declarations
	//
   CGPManager = class;
   CGPSoundSystem = class;
   CGPSoundSegments = class;
	CGPSoundSegment = class;
   CGPEqualizer = class;
	CGPOptions = class;
	CGPFile = class;

	// Event declarations
   CGPOnPlayStateChangedEvent = procedure(Sender: TObject; PlayState: CGPPlayState) of object;
   CGPOnPlaySoundSegmentEvent = procedure(Sender: TObject; SoundSegment: CGPSoundSegment) of object;
	CGPOnPlayingEvent = procedure(Sender: TObject; Position: currency) of object;

	//---------------------------------------------------------------------------
	// The main manager
	//---------------------------------------------------------------------------

	 CGPManager = class
		// Private declarations
		private
           m_soundSystem: CGPSoundSystem;
			m_soundSegments: CGPSoundSegments;
			m_options: CGPOptions;
			m_gpFile: CGPFile;
			m_activeSoundSegment: CGPSoundSegment;
           m_timer: TTimer;
       private
       	m_onPlayStateChangedEvent: CGPOnPlayStateChangedEvent;
           m_onPlaySoundSegmentEvent: CGPOnPlaySoundSegmentEvent;
			m_onPlayingEvent: CGPOnPlayingEvent;
			m_onActiveSoundSegmentChhangedEvent: TNotifyEvent;
		private
       	procedure onTimer(Sender: TObject);
		private
			function getSoundSystem: CGPSoundSystem;
			function getSoundSegments: CGPSoundSegments;
			function getOptions: CGPOptions;
			function getGPFile: CGPFile;
			function getHasActiveSoundSegment: boolean;
			function getActiveSoundSegment: CGPSoundSegment;
			procedure setActiveSoundSegment(val: CGPSoundSegment);
		private
			procedure updateSoundSegment;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create;
			destructor Destroy; override;
		public
			procedure Reset;
			function AddSoundSegment(fileName: string): CGPSoundSegment;
			procedure PlaySoundSegment(idx: integer);
           procedure EncodeSoundSegment(idx: integer);
			procedure PlayPauseSoundSegment;
			procedure Stop;
			function GetEncoderCommandLine(idx: integer; soundSegment: CGPSoundSegment): string;
			function GetEncoderFileName(idx: integer; soundSegment: CGPSoundSegment): string;
		public
			property SoundSystem: CGPSoundSystem read getSoundSystem;
			property SoundSegments: CGPSoundSegments read getSoundSegments;
			property Options: CGPOptions read getOptions;
			property GPFile: CGPFile read getGPFile;
			property HasActiveSoundSegment: boolean read getHasActiveSoundSegment;
			property ActiveSoundSegment: CGPSoundSegment read getActiveSoundSegment write setActiveSoundSegment;
		public
			property OnActiveSoundSegmentChanged: TNotifyEvent read m_onActiveSoundSegmentChhangedEvent write m_onActiveSoundSegmentChhangedEvent;
       	property OnPlaySoundSegment: CGPOnPlaySoundSegmentEvent read m_onPlaySoundSegmentEvent write m_onPlaySoundSegmentEvent;
       	property OnPlayStateChanged: CGPOnPlayStateChangedEvent read m_onPlayStateChangedEvent write m_onPlayStateChangedEvent;
			property OnPlaying: CGPOnPlayingEvent read m_onPlayingEvent write m_onPlayingEvent;
	end;

	//---------------------------------------------------------------------------
	// The sound system
	//---------------------------------------------------------------------------

	 CGPSoundSystem = class
		// Private declarations
		private
       	m_manager: CGPManager;
			m_initialized: boolean;
           m_info: BASS_CHANNELINFO;
           m_equalizerFx: dword;
			m_sampleRate: integer;
			m_volume: integer;
			m_tempo: integer;
			m_pitch: integer;
			m_panning: integer;
			m_stream: HStream;
           m_equalizer: CGPEqualizer;
       private
       	function getEqualizerInitialized: boolean;
       	function getHasStream: boolean;
			function getPlaying: boolean;
			function getPaused: boolean;
			function getStopped: boolean;
			function getChannelLength: integer;
			function getPosition: currency;
			procedure setPosition(val: currency);
			function getSampleRate: integer;
			procedure setSampleRate(val: integer);
			function getVolume: integer;
			procedure setVolume(val: integer);
			function getTempo: integer;
			procedure setTempo(val: integer);
			function getPitch: integer;
			procedure setPitch(val: integer);
			function getPanning: integer;
			procedure setPanning(val: integer);
			function getPlayState: CGPPlayState;
           function getEqualizer: CGPEqualizer;
		private
			function createStream(filename: string; const forDecode: boolean = false): HSTREAM;
       	procedure freeStream;
			procedure error(msg: string);
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CGPManager);
			destructor Destroy; override;
		public
			function Initialize: boolean;
           function Deinitialize: boolean;
           procedure PlayFile(filename: string);
			procedure EncodeSoundSegment(idx: integer; soundSegment: CGPSoundSegment);
           procedure Start;
           procedure Stop;
           procedure Pause;
       	procedure SetEqualizerParameters(eqParameters: BASS_BFX_PEAKEQ);
           function GetEqualizerParameters(var eqParameters: BASS_BFX_PEAKEQ): boolean;
		public
           property SampleRate: integer read getSampleRate write setSampleRate;
			property Volume: integer read getVolume write setVolume;
           property Tempo: integer read getTempo write setTempo;
           property Pitch: integer read getPitch write setPitch;
           property Panning: integer read getPanning write setPanning;
           property ChannelLength: integer read getChannelLength;
           property Position: currency read getPosition write setPosition;
			property HasStream: boolean read getHasStream;
			property Playing: boolean read getPlaying;
           property Stopped: boolean read getStopped;
           property Paused: boolean read getPaused;
           property PlayState: CGPPlayState read getPlayState;
           property Equalizer: CGPEqualizer read getEqualizer;
           property EqualizerInitialized: boolean read getEqualizerInitialized;
	end;

	//---------------------------------------------------------------------------
	// The sound segments
	//---------------------------------------------------------------------------

	 CGPSoundSegments = class
		// Private declarations
		private
       	m_manager: CGPManager;
			m_segments: TObjectList;
       private
       	function getSoundSegment(idx: integer): CGPSoundSegment;
           function getCount: integer;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CGPManager);
			destructor Destroy; override;
       public
       	procedure Reset;
           procedure Delete(idx: integer); 
           function AddSoundSegment: CGPSoundSegment;
           function CopyAndAddSoundSegment(segment: CGPSoundSegment): CGPSoundSegment;
       public
       	property SoundSegment[idx: integer]: CGPSoundSegment read getSoundSegment; default;
           property Count: integer read getCount;
	end;

	//---------------------------------------------------------------------------
	// The sound segment
	//---------------------------------------------------------------------------

	 CGPSoundSegment = class
		// Private declarations
		private
       	m_segments: CGPSoundSegments;
			m_filename: string;
			m_description: string;
			m_loop: boolean;
           m_loopStart: currency;
           m_loopEnd: currency;
			m_seconds: integer;
			m_sampleRate: integer;
			m_volume: integer;
			m_tempo: integer;
			m_pitch: integer;
			m_panning: integer;
		private
       	function getPlaytime: currency;
			function getCanLoop: boolean;
			function getShortFileName: string;
			function getAsString: string;
			procedure setAsString(val: string);
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(mySegments: CGPSoundSegments);
			destructor Destroy; override;
       public
			procedure Reset;
		public
			property FileName: string read m_fileName write m_fileName;
			property ShortFileName: string read getShortFileName;
			property Description: string read m_description write m_description;
			property CanLoop: boolean read getCanLoop;
			property Loop: boolean read m_loop write m_loop;
			property LoopStart: currency read m_loopStart write m_loopStart;
			property LoopEnd: currency read m_loopEnd write m_loopEnd;
			property Seconds: integer read m_seconds write m_seconds;
           property Playtime: currency read getPlayTime;
			property SampleRate: integer read m_sampleRate write m_sampleRate;
			property Volume: integer read m_volume write m_volume;
			property Tempo: integer read m_tempo write m_tempo;
			property Pitch: integer read m_pitch write m_pitch;
			property Panning: integer read m_panning write m_panning;
			property AsString: string read getAsString write setAsString;
	end;

	//---------------------------------------------------------------------------
	// The equalizer
	//---------------------------------------------------------------------------

	 CGPEqualizer = class
		// Private declarations
		private
       	m_soundSystem: CGPSoundSystem;
           m_parameters: BASS_BFX_PEAKEQ;
           m_bands: array[0..9] of integer;
       private
       	function getBand1Gain: integer;
           procedure setBand1Gain(val: integer);
       	function getBand2Gain: integer;
           procedure setBand2Gain(val: integer);
       	function getBand3Gain: integer;
           procedure setBand3Gain(val: integer);
       	function getBand4Gain: integer;
           procedure setBand4Gain(val: integer);
       	function getBand5Gain: integer;
           procedure setBand5Gain(val: integer);
       	function getBand6Gain: integer;
           procedure setBand6Gain(val: integer);
       	function getBand7Gain: integer;
           procedure setBand7Gain(val: integer);
       	function getBand8Gain: integer;
           procedure setBand8Gain(val: integer);
       	function getBand9Gain: integer;
           procedure setBand9Gain(val: integer);
       	function getBand10Gain: integer;
           procedure setBand10Gain(val: integer);
           function getAsString: string;
			procedure setAsString(val: string);
       private
       	procedure initBand(number: integer; centerFrequency: float);
           procedure updateBand(number: integer; gain: integer);
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(mySoundSystem: CGPSoundSystem);
			destructor Destroy; override;
       public
       	procedure Reset;
           procedure Initialize;
       public
       	property SoundSystem: CGPSoundSystem read m_soundSystem;
           property Band1Gain: integer read getBand1Gain write setBand1Gain;
           property Band2Gain: integer read getBand2Gain write setBand2Gain;
           property Band3Gain: integer read getBand3Gain write setBand3Gain;
           property Band4Gain: integer read getBand4Gain write setBand4Gain;
           property Band5Gain: integer read getBand5Gain write setBand5Gain;
           property Band6Gain: integer read getBand6Gain write setBand6Gain;
           property Band7Gain: integer read getBand7Gain write setBand7Gain;
           property Band8Gain: integer read getBand8Gain write setBand8Gain;
           property Band9Gain: integer read getBand9Gain write setBand9Gain;
           property Band10Gain: integer read getBand10Gain write setBand10Gain;
           property AsString: string read getAsString write setAsString;
	end;

	//---------------------------------------------------------------------------
	// The options
	//---------------------------------------------------------------------------

	CGPOptions = class
		// Private declarations
		private
			m_manager: CGPManager;
			m_file: TIniFile;
			m_left: integer;
			m_top: integer;
			m_width: integer;
			m_height: integer;
           m_language: string;
			m_lastOpenFilename: string;
           m_lastOpenFiles: string;
           m_currentMP3Directory: string;
			m_currentGPFDirectory: string;
			m_encoder: string;
			m_encoderCommandLine: string;
			m_encoderOutputDirectory: string;
			m_encoderDefaults: TStringList;
			m_keepFileHistory: integer;
		private
			function getOptionsFileName: string;
			function getCanExport: boolean;
			function getEncoderDefaultCommandLineByName(name: string): string;
			function getHasEncoder(name: string): boolean;
			function getEncoderOutputDirectory: string;
			procedure setEncoderOutputDirectory(val: string);
		private
			procedure saveOptions;
			procedure loadOptions;
			procedure initEncoderDefaults;
			procedure saveEncoderDefaults;
			procedure loadEncoderDefaults;
		// Public declarations
		public
			constructor Create(myManager: CGPManager);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure Load;
			procedure Save;
		public
			property OptionsFileName: string read getOptionsFileName;
			property Left: integer read m_left write m_left;
			property Top: integer read m_top write m_top;
			property Width: integer read m_width write m_width;
			property Height: integer read m_height write m_height;
           property Language: string read m_language write m_language;
			property LastOpenFileName: string read m_lastopenfilename write m_lastopenfilename;
           property LastOpenFiles: string read m_lastOpenFiles write m_lastOpenFiles;
           property CurrentMP3Directory: string read m_currentMP3Directory write m_currentMP3Directory;
			property CurrentGPFDirectory: string read m_currentGPFDirectory write m_currentGPFDirectory;
			property EncoderDefaultCommandLineByName[name: string]: string read getEncoderDefaultCommandLineByName;
			property HasEncoder[name: string]: boolean read getHasEncoder;
			property CanExport: boolean read getCanExport;
			property Encoder: string read m_encoder write m_encoder;
			property EncoderCommandLine: string read m_encoderCommandLine write m_encoderCommandLine;
			property EncoderOutputDirectory: string read getEncoderOutputDirectory write setEncoderOutputDirectory;
			property KeepFileHistory: integer read m_keepFileHistory write m_keepFileHistory;
	end;

	//---------------------------------------------------------------------------
	// The guitar practice file
	//---------------------------------------------------------------------------

	CGPFile = class
		// Private declarations
		private
			m_manager: CGPManager;
			m_file: TIniFile;
			m_name: string;
			m_title: string;
		private
			procedure saveFileData;
			procedure loadFileData;
		// Public declarations
		public
			constructor Create(myManager: CGPManager);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure Load(fileName: string);
			procedure Save(fileName: string);
		public
			property Name: string read m_name;
			property Title: string read m_title write m_title;
	end;

implementation

uses
	CGPUtils, CGPConst, CFileClasses, UFrmGuitarPracticeProgress;

//------------------------------------------------------------------------------
// Implementation of CGPManager.pas
//------------------------------------------------------------------------------
{$I CGPManager.pas}

//------------------------------------------------------------------------------
// Implementation of CGPSoundSystem.pas
//------------------------------------------------------------------------------
{$I CGPSoundSystem.pas}

//------------------------------------------------------------------------------
// Implementation of CGPSoundSegments.pas
//------------------------------------------------------------------------------
{$I CGPSoundSegments.pas}

//------------------------------------------------------------------------------
// Implementation of CGPSoundSegment.pas
//------------------------------------------------------------------------------
{$I CGPSoundSegment.pas}

//------------------------------------------------------------------------------
// Implementation of CGPEqualizer.pas
//------------------------------------------------------------------------------
{$I CGPEqualizer.pas}

//------------------------------------------------------------------------------
// Implementation of CGPOptions.pas
//------------------------------------------------------------------------------
{$I CGPOptions.pas}

//------------------------------------------------------------------------------
// Implementation of CGPFile.pas
//------------------------------------------------------------------------------
{$I CGPFile.pas}

end.
