//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPManager.
//
//	Description:  	This unit implements the guitar practice manager class
//
//
//
// Author(s):		FeMann
// Created:		30.08.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//--------------------------------------------------------------------------
// Constructor and Destructor
//--------------------------------------------------------------------------

// Constructor
//
constructor CGPManager.Create;
begin
	inherited;

   // Create the timer
   m_timer := TTimer.Create(nil);
   m_timer.Enabled := false;
   m_timer.Interval := 5;
   m_timer.OnTimer := onTimer;

   // Reset the manager
   Reset;
end;

// Destructor
//
destructor CGPManager.Destroy;
begin
	try
   	// Stop the timer
       m_timer.Enabled := false;

   	// Free the timer
		FreeAndNil(m_timer);

		// Free the gp file
		if Assigned(m_gpFile) then
			FreeAndNil(m_gpFile);

		// Free sound system
		if Assigned(m_soundSystem) then
			FreeAndNil(m_soundSystem);

		// Free sound segments
		if Assigned(m_soundSegments) then
			FreeAndNil(m_soundSegments);

		// Free the options
		if Assigned(m_options) then
			FreeAndNil(m_options);
	finally
   	inherited;
   end;
end;

//--------------------------------------------------------------------------
// Public methods
//--------------------------------------------------------------------------

// Reset the manager
//
procedure CGPManager.Reset;
begin
	// Reset the sound segment
	ActiveSoundSegment := nil;
end;

// Add sound segment
//
function CGPManager.AddSoundSegment(fileName: string): CGPSoundSegment;
begin
	// Add new sound segment
	result := SoundSegments.AddSoundSegment;

	// Setup filename
	result.FileName := fileName;
	result.Description := result.ShortFileName;
end;

// Load the sound segment
//
procedure CGPManager.PlaySoundSegment(idx: integer);
begin
	// Get active sound segment
	ActiveSoundSegment := SoundSegments[idx];

	// Check if file exists
	if FileExists(ActiveSoundSegment.FileName) then
   begin
       // Play the file
		SoundSystem.PlayFile(ActiveSoundSegment.FileName);

		// Update the active sound segment
		updateSoundSegment;

       // Fire play sound segment event
   	if Assigned(m_onPlaySoundSegmentEvent) then
			m_onPlaySoundSegmentEvent(self,ActiveSoundSegment);

		// Check loop and set start position
		if ActiveSoundSegment.CanLoop then
			SoundSystem.Position := ActiveSoundSegment.LoopStart;

      	// Fire playing event
   	if Assigned(m_onPlayingEvent) then
   		m_onPlayingEvent(self,SoundSystem.Position);

       // Enable the timer
       m_timer.Enabled := true;

      	// Fire play state changed event
   	if Assigned(m_onPlayStateChangedEvent) then
   		m_onPlayStateChangedEvent(self,SoundSystem.PlayState);
   end;
end;

// Load the sound segment
//
procedure CGPManager.EncodeSoundSegment(idx: integer);
begin
	// Get active sound segment
	ActiveSoundSegment := SoundSegments[idx];

	// Check if file exists
	if FileExists(ActiveSoundSegment.FileName) then
	begin
		// Encode the sound segment
		SoundSystem.EncodeSoundSegment(idx,ActiveSoundSegment);
	end;
end;

// Play or pause the sound segment
//
procedure CGPManager.PlayPauseSoundSegment;
begin
	// Validate
	if not HasActiveSoundSegment then
		exit;

   // Pause or play the file
   if SoundSystem.Playing then
   	SoundSystem.Pause
   else
   	SoundSystem.Start;

   // Fire play state changed event
   if Assigned(m_onPlayStateChangedEvent) then
   	m_onPlayStateChangedEvent(self,SoundSystem.PlayState);

   // Enable/Disable the timer
   m_timer.Enabled := SoundSystem.Playing;
end;

// Stop the manager
//
procedure CGPManager.Stop;
begin
   // Disable the timer
   m_timer.Enabled := false;
end;

// Get the encoder command line
//
function CGPManager.GetEncoderCommandLine(idx: integer; soundSegment: CGPSoundSegment): string;
var
	outputFileName: string;
	cmd: string;
begin
	// Get the output file name
	outputFileName := '"' +  GetEncoderFileName(idx,soundSegment) + '"';

	// Get the command
	cmd := Options.Encoder + ' ' + Options.EncoderCommandLine;

	// Replace the output filename
	result := StringReplace(cmd,'%OUTPUTFILENAME%',outputFileName,[rfReplaceAll, rfIgnoreCase]);
end;

// Get the encoder filename
//
function CGPManager.GetEncoderFileName(idx: integer; soundSegment: CGPSoundSegment): string;
var
	soundSegmentDescription: string;
begin
	// Cut possible extension in sound segment
	if (Pos('.',soundSegment.Description) > 0) then
		soundSegmentDescription := Copy(soundSegment.Description,1,Pos('.',soundSegment.Description) - 1)
	else
		soundSegmentDescription := soundSegment.Description;

	// Get the output file name
	result := Options.EncoderOutputDirectory + '[' + IntToStr(idx + 1) + '] - ' + soundSegmentDescription + ' - (T' + IntToStr(SoundSegment.Tempo) + ' P' + IntToStr(SoundSegment.Pitch) + ')';
end;

//--------------------------------------------------------------------------
// Private methods
//--------------------------------------------------------------------------

// Update the sound segment
//
procedure CGPManager.updateSoundSegment;
begin
	// Nothing more to do
	if not HasActiveSoundSegment then
		exit;

	// Update the segments seconds
	ActiveSoundSegment.Seconds := SoundSystem.ChannelLength;

	// Update sample rate
	if (ActiveSoundSegment.SampleRate = -1) then
		ActiveSoundSegment.SampleRate := SoundSystem.SampleRate;

	// Update volume
	if (ActiveSoundSegment.Volume = -1) then
		ActiveSoundSegment.Volume := SoundSystem.Volume;

	// Update tempo
	if (ActiveSoundSegment.Tempo = -1) then
		ActiveSoundSegment.Tempo := SoundSystem.Tempo;

	// Update pitch
	if (ActiveSoundSegment.Pitch = -1) then
		ActiveSoundSegment.Pitch := SoundSystem.Pitch;

	// Update panning
	if (ActiveSoundSegment.Panning = -1) then
		ActiveSoundSegment.Panning := SoundSystem.Panning;
end;

//--------------------------------------------------------------------------
// Event methods
//--------------------------------------------------------------------------

// On timer notification
//
procedure CGPManager.onTimer(Sender: TObject);
begin
	// Validate
	if not HasActiveSoundSegment then
		exit;

	// Check if loop necessary
	if ActiveSoundSegment.CanLoop then
   begin
   	// Check position boundaries
		if (SoundSystem.Position >= ActiveSoundSegment.LoopEnd) or (SoundSystem.Position < ActiveSoundSegment.LoopStart) then
       begin
       	// Bugfix for loop bug
           if (ActiveSoundSegment.LoopStart > 0) then
				// Set position to loop start
				SoundSystem.Position := ActiveSoundSegment.LoopStart
           else
           	SoundSystem.Position := 0.01;
       end;
	end;

	// Fire playing event
   if Assigned(m_onPlayingEvent) then
   	m_onPlayingEvent(self,SoundSystem.Position);
end;

//--------------------------------------------------------------------------
// Property methods
//--------------------------------------------------------------------------

// Get the sound system
//
function CGPManager.getSoundSystem: CGPSoundSystem;
begin
	// Create sound system on demand
	if not Assigned(m_soundSystem) then
   begin
       // Create sound system
   	m_soundSystem := CGPSoundSystem.Create(self);

       // Initialize sound system
       m_soundSystem.Initialize;
   end;

   // Return sound system instance
   result := m_soundSystem;
end;

// Get the sound segments
//
function CGPManager.getSoundSegments: CGPSoundSegments;
begin
	// Create sound segments on demand
	if not Assigned(m_soundSegments) then
		m_soundSegments := CGPSoundSegments.Create(self);

	// Return sound segments instance
	result := m_soundSegments;
end;

// Determine if there is an active sound segment
//
function CGPManager.getHasActiveSoundSegment: boolean;
begin
	result := (ActiveSoundSegment <> nil);
end;

// Get the active sound segment
//
function CGPManager.getActiveSoundSegment: CGPSoundSegment;
begin
	result := m_activeSoundSegment;
end;

// Set the active sound segment
//
procedure CGPManager.setActiveSoundSegment(val: CGPSoundSegment);
begin
	// Set the sound segment
	m_activeSoundSegment := val;

	// Fire active sound segment changed event
	if Assigned(m_onActiveSoundSegmentChhangedEvent) then
		m_onActiveSoundSegmentChhangedEvent(self);
end;

// Get options
//
function CGPManager.getOptions: CGPOptions;
begin
	// Create options on demand
	if not Assigned(m_options) then
		m_options := CGPOptions.Create(self);

	// Return options instance
	result := m_options;
end;

// Get the gp file
//
function CGPManager.getGPFile: CGPFile;
begin
	// Create gp file on demand
	if not Assigned(m_gpFile) then
		m_gpFile := CGPFile.Create(self);

	// Return gp file instance
	result := m_gpFile;
end;
