//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPSoundSystem.
//
//	Description:  	This unit implements the guitar practice sound system
//
//
//
// Author(s):		FeMann
// Created:		30.08.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//--------------------------------------------------------------------------
// Constructor and Destructor
//--------------------------------------------------------------------------

// Constructor
//
constructor CGPSoundSystem.Create(myManager: CGPManager);
begin
	inherited Create;

   // Set backward reference to manager
   m_manager := myManager;

   // Initialize variables
	m_initialized := false;
	m_sampleRate := 44100;
   m_volume := 100;
   m_tempo := 0;
   m_pitch := 0;
   m_panning := 0;
   m_stream := 0;
   m_equalizerFx := 0;
end;

// Destructor
//
destructor CGPSoundSystem.Destroy;
begin
	try
       // Free the equalizer
       if Assigned(m_equalizer) then
       	FreeAndNil(m_equalizer);

   	// Deinitialize the sound system
       Deinitialize;

       // Reset backward reference to manager
       m_manager := nil;
   finally
   	inherited;
   end;
end;

//--------------------------------------------------------------------------
// Public methods
//--------------------------------------------------------------------------

// Initialize the sound system
//
function CGPSoundSystem.Initialize: boolean;
begin
	// Initialize result
   result := false;

   // Get the bass version
   if HIWORD(BASS_GetVersion) <> BASSVERSION then
   begin
    	error('BASS version 2.4 could not be loaded');
   	exit;
   end;

  	// Get the bass fx version
  	If HIWORD(BASS_FX_GetVersion) <> BASSVERSION then
   begin
    	error('BASS FX version 2.4 could not be loaded');
   	exit;
	end;

   // Initialize bass -> Default sound card
	if not BASS_Init(-1, m_sampleRate, 0, Application.Handle, nil) then
	begin
   	error('Can''t initialize digital sound system');
   	exit;
   end;

   // Start the system
	m_initialized := true;
   result := m_initialized;
end;

// Deinitialize the sound system
//
function CGPSoundSystem.Deinitialize: boolean;
begin
	// Initialize result
	result := m_initialized;

   if not m_initialized then
   	exit;

   // Stop the stream
   Stop;

	// Stop the system
	BASS_Stop;

   // Free stream
   freeStream;

   // Close the system
	BASS_Free;

   m_initialized := false;
   result := m_initialized;
end;

// Play the sound
//
procedure CGPSoundSystem.PlayFile(filename: string);
var
	stream: HSTREAM;
begin
	// Check initialization
   if not m_initialized then
   	exit;

   // Check if file exists
   if (filename = '') or not FileExists(filename) then
   	exit;

   // Free unused streams
   freeStream;

	// Create stream
   stream := createStream(filename);

   // Initialize the equalizer
   Equalizer.Initialize;

   // Play channel
   if not BASS_ChannelPlay(stream, true) then
   	Error('Error playing stream!');
end;

// Encode the sound segment
//
procedure CGPSoundSystem.EncodeSoundSegment(idx: integer; soundSegment: CGPSoundSegment);
var
	stream: HSTREAM;
   encodingFinished: boolean;
   buffer: array [0..10000] of byte;
	bytesEncoded: integer;
	bufferRead: integer;
	startPos: currency;
	endPos: currency;
	bytesToEncode: integer;
	cmd: string;
	frmProgress: TFrmGuitarPracticeProgress;
begin
	// Check initialization
   if not m_initialized then
   	exit;

   // Check if file exists
   if (soundSegment.Filename = '') or not FileExists(soundSegment.Filename) then
		exit;
		
   // Reset encoding finished
   encodingFinished := false;
       
   // Free unused streams
   freeStream;

	// Create stream
   stream := createStream(soundSegment.Filename,true);

   // Initialize the equalizer
   Equalizer.Initialize;

	// Calculate start position
	if soundSegment.CanLoop then
		startPos := BASS_ChannelSeconds2Bytes(m_stream, soundSegment.LoopStart) 
	else
		startPos := 0;

	// Calculate end position
	if soundSegment.CanLoop then
		endPos := BASS_ChannelSeconds2Bytes(m_stream, soundSegment.LoopEnd) 
	else
		endPos := BASS_ChannelGetLength(stream, BASS_POS_BYTE);

	// Calculate the bytes to encode
	bytesToEncode := Round(endPos - startPos);

	// Reset already encoded bytes
	bytesEncoded := 0;

	// Set stream position
	BASS_ChannelSetPosition(stream, Trunc(startPos), BASS_POS_BYTE);
		
	// Update sound system values
	SampleRate := soundSegment.SampleRate;
	Tempo := soundSegment.Tempo;
	Pitch := soundSegment.Pitch;

	// Get the file name
	cmd := m_manager.GetEncoderCommandLine(idx,soundSegment);
	
	// Start the encoder
	if (BASS_Encode_Start(stream, PWideChar(cmd), BASS_ENCODE_AUTOFREE or BASS_UNICODE, nil, nil) = 0) then
	begin
		 // Free the stream
		 freeStream;
		 
		 // Set error
		 error('Could not start encoder. Please check the encoder options!');

		 // Nothing more to do
		 exit;
	end;

	// Create the progress form
	frmProgress := TFrmGuitarPracticeProgress.Create(Application.MainForm);
	try
		// Show the form
		frmProgress.Execute(bytesToEncode);

		// Update the file name
		frmProgress.OutputDirectory := m_manager.Options.EncoderOutputDirectory;
		frmProgress.OutputFileName := m_manager.GetEncoderFileName(idx,soundSegment);

		// Process the messages
		Application.ProcessMessages;

		// Stream the stream to the encoder
		while (BASS_ChannelIsActive(stream) > 0) and not encodingFinished do
		begin
			// Clear the buffer
			FillChar(buffer,SizeOf(buffer),#0);
		
			// Check buffer length
			if (bytesToEncode > SizeOf(buffer)) then
				bufferRead := SizeOf(buffer) - 1			
			else
				bufferRead := bytesToEncode;  
						
			// Read the buffer
			BASS_ChannelGetData(stream, @buffer, bufferRead);

			// Decrease bytes to encode counter
			dec(bytesToEncode,bufferRead);

			// Increase already encoded bytes counter
			inc(bytesEncoded,bufferRead);
		
			// Check for encoding finished
			encodingFinished := (bytesToEncode <= 0);
		 
			// Update the form
			frmProgress.Progress := bytesEncoded;
		 
			// Process the messages
			Application.ProcessMessages;
		end;
	finally
		// Free the stream
		freeStream;

		// Hide and free the form
		frmProgress.Hide;
		FreeAndNil(frmProgress);
	end;
end;

// Start the sound
//
procedure CGPSoundSystem.Start;
begin
	// Start stream
   if HasStream then
		BASS_Start;
end;

// Stop the sound
//
procedure CGPSoundSystem.Stop;
begin
	// Stop stream
   if HasStream then
   	BASS_ChannelStop(m_stream);
end;

// Pause the sound
//
procedure CGPSoundSystem.Pause;
begin
	// Pause stream
   if HasStream then
		BASS_Pause;
end;

// Set the equalizer parameters
//
procedure CGPSoundSystem.SetEqualizerParameters(eqParameters: BASS_BFX_PEAKEQ);
begin
	// Check for stream and equalizer
	if HasStream and EqualizerInitialized then
   	BASS_FXSetParameters(m_equalizerFx, @eqParameters);
end;

// Get the equalizer parameters
//
function CGPSoundSystem.GetEqualizerParameters(var eqParameters: BASS_BFX_PEAKEQ): boolean;
begin
	// Initialize result
   result := false;

	// Check for stream and equalizer
	if HasStream and EqualizerInitialized then
   begin
   	// Get the equalizer parameters
   	if BASS_FXGetParameters(m_equalizerFx, @eqParameters) then
			result := true;
   end;
end;

//--------------------------------------------------------------------------
// Private methods
//--------------------------------------------------------------------------

// Output error message
//
procedure CGPSoundSystem.error(msg: string);
var
	s: string;
begin
	s := msg + #13#10 + '(Error code: ' + IntToStr(BASS_ErrorGetCode) + ')';
  	MessageBox(Application.Handle, PChar(s), 'Error', MB_ICONERROR or MB_OK);
end;

// Create the stream
//
function CGPSoundSystem.createStream(filename: string; const forDecode: boolean = false): HSTREAM;
var
	flags: DWORD;
begin
   // Initialize result
	result := 0;

	// Create new stream
	m_stream := BASS_StreamCreateFile(false, PChar(filename), 0, 0, BASS_STREAM_DECODE or BASS_UNICODE);

	// Check if everything is ok
   if not HasStream then
   begin
   	error('Error creating stream!');
       exit;
   end;

  	// Get the channel info
  	BASS_ChannelGetInfo(m_stream, m_info);

   // Setup frequency
	m_sampleRate := m_info.freq;

	if forDecode then
		flags := BASS_SAMPLE_LOOP or BASS_FX_FREESOURCE or BASS_STREAM_DECODE
	else
		flags := BASS_SAMPLE_LOOP or BASS_FX_FREESOURCE;
	
	// Create a new stream - decoded & resampled
	m_stream := BASS_FX_TempoCreate(m_stream, flags);

   // Check if everything is ok
	if not HasStream then
   begin
   	error('Couldn''t create a resampled stream!');
       exit;
   end;

   // Remove the equalizer effect
   if EqualizerInitialized then
   begin
       // remove effect from channel
   	BASS_ChannelRemoveFX(m_stream, m_equalizerFx);

       // Reset equalizer effect
       m_equalizerFx := 0;
   end;

	// Set peaking equalizer effect with no bands
	m_equalizerFx := BASS_ChannelSetFX(m_stream, BASS_FX_BFX_PEAKEQ, 0);

   // Return stream
   result := m_stream;
end;

// Free the stream
//
procedure CGPSoundSystem.FreeStream;
begin
	// Determine if stream is still active
	if HasStream then
   begin
       // Stop the stream
       Stop;

		// Free the stream
   	if (BASS_ChannelIsActive(m_stream) <> BASS_ACTIVE_PLAYING) then
           BASS_StreamFree(m_stream);

       // Reset stream
       m_stream := 0;

       // Reset equalizer
       m_equalizerFx := 0;
   end;
end;

//--------------------------------------------------------------------------
// Property methods
//--------------------------------------------------------------------------

// Determine if equalizer initialized
//
function CGPSoundSystem.getEqualizerInitialized: boolean;
begin
	result := (m_equalizerFx <> 0);
end;

// Determine if stream present
//
function CGPSoundSystem.getHasStream: boolean;
begin
	result := (m_stream <> 0);
end;

// Get the channel length
//
function CGPSoundSystem.getChannelLength: integer;
begin
	// Initialize result
   result := 0;

	if HasStream then
		result := Round(BASS_ChannelBytes2Seconds(m_stream, BASS_ChannelGetLength(m_stream, BASS_POS_BYTE)));
end;

// Determine if channel is playing
//
function CGPSoundSystem.getPlaying: boolean;
begin
	result := HasStream and (BASS_ChannelIsActive(m_stream) = BASS_ACTIVE_PLAYING);
end;

// Determine if channel is paused
//
function CGPSoundSystem.getPaused: boolean;
begin
	result := HasStream and (BASS_ChannelIsActive(m_stream) = BASS_ACTIVE_PAUSED);
end;

// Determine if channel is stopped
//
function CGPSoundSystem.getStopped: boolean;
begin
	result := HasStream and (BASS_ChannelIsActive(m_stream) = BASS_ACTIVE_STOPPED);
end;

// Determine the play state
//
function CGPSoundSystem.getPlayState: CGPPlayState;
begin
	// Playing
	if Playing then
   	result := psPlaying
   // Stopped
   else if Stopped then
   	result := psStopped
   else if Paused then
   	result := psPaused
   else
   	result := psUnknown;
end;

// Get the position
//
function CGPSoundSystem.getPosition: currency;
begin
	// Initialize result
   result := 0;

   if HasStream then
		result := BASS_ChannelBytes2Seconds(m_stream, BASS_ChannelGetPosition(m_stream, BASS_POS_BYTE));
end;

// Set the position
//
procedure CGPSoundSystem.setPosition(val: currency);
begin
	if HasStream then
		BASS_ChannelSetPosition(m_stream, BASS_ChannelSeconds2Bytes(m_stream, val), BASS_POS_BYTE);
end;

// Get the sample rate
//
function CGPSoundSystem.getSampleRate: integer;
var
	frq: single;
begin
	// Initialize result
   result := 0;

   // Get the frequency
	if HasStream then
   begin
		if BASS_ChannelGetAttribute(m_stream, BASS_ATTRIB_TEMPO_FREQ, frq) then
   	begin
   		m_sampleRate := Trunc(frq);
       	result := m_sampleRate;
   	end;
   end;
end;

// Set the sample rate
//
procedure CGPSoundSystem.setSampleRate(val: integer);
begin
   // Set the frequency
   if HasStream then
   begin
		if BASS_ChannelSetAttribute(m_stream, BASS_ATTRIB_TEMPO_FREQ, val) then
   		m_sampleRate := val;
   end;
end;

// Get the volume
//
function CGPSoundSystem.getVolume: integer;
var
	vol: single;
begin
	// Initialize result
   result := 0;

	// Get volume
	if HasStream then
   begin
		if BASS_ChannelGetAttribute(m_stream, BASS_ATTRIB_VOL, vol) then
       begin
       	m_volume := Round(vol * 100);
           result := m_volume;
       end;
   end;
end;

// Set the volume
//
procedure CGPSoundSystem.setVolume(val: integer);
begin
   // Set volume
	if HasStream then
   begin
		if BASS_ChannelSetAttribute(m_stream, BASS_ATTRIB_VOL, val / 100) then
       	m_volume := val;
   end;
end;

// Get the tempo
//
function CGPSoundSystem.getTempo: integer;
var
	tempo: single;
begin
	// Initialize result
   result := 0;

	// Get tempo
	if HasStream then
   begin
		if BASS_ChannelGetAttribute(m_stream, BASS_ATTRIB_TEMPO, tempo) then
       begin
       	m_tempo := Trunc(tempo);
           result := m_tempo;
       end;
   end;
end;

// Set the tempo
//
procedure CGPSoundSystem.setTempo(val: integer);
begin
   // Set tempo
	if HasStream then
   begin
		if BASS_ChannelSetAttribute(m_stream, BASS_ATTRIB_TEMPO, val) then
       	m_tempo := val;
   end;
end;

// Get the pitch
//
function CGPSoundSystem.getPitch: integer;
var
	pitch: single;
begin
	// Initialize result
   result := 0;

	// Get tempo
	if HasStream then
   begin
		if BASS_ChannelGetAttribute(m_stream, BASS_ATTRIB_TEMPO_PITCH, pitch) then
       begin
       	m_pitch := Trunc(pitch);
           result := m_pitch;
       end;
   end;
end;

// Set the tempo
//
procedure CGPSoundSystem.setPitch(val: integer);
begin
   // Set tempo
	if HasStream then
   begin
		if BASS_ChannelSetAttribute(m_stream, BASS_ATTRIB_TEMPO_PITCH, val) then
       	m_pitch := val;
   end;
end;

// Get the panning
//
function CGPSoundSystem.getPanning: integer;
var
	pan: single;
begin
	// Initialize result
   result := 0;

	// Get panning
	if HasStream then
   begin
		if BASS_ChannelGetAttribute(m_stream, BASS_ATTRIB_PAN, pan) then
       begin
       	m_panning := Round(pan * 100);
           result := m_panning;
       end;
   end;
end;

// Set the panning
//
procedure CGPSoundSystem.setPanning(val: integer);
begin
   // Set panning
	if HasStream then
   begin
		if BASS_ChannelSetAttribute(m_stream, BASS_ATTRIB_PAN, val / 100) then
       	m_panning := val;
   end;
end;

// Get the equalizer
//
function CGPSoundSystem.getEqualizer: CGPEqualizer;
begin
   // Create equalizer on demand
	if not Assigned(m_equalizer) then
		m_equalizer := CGPEqualizer.Create(self);

   // Return equalizer instance
   result := m_equalizer;
end;
