//------------------------------------------------------------------------------
// File classes
//------------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CLastOpenFiles.
//
//	Description:  	This unit implements the last open files class
//
//
//
// Author(s):		FeMann
// Created:		04.10.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CLastOpenFiles.Create;
begin
	inherited;

   // Create file container
   m_files := TObjectList.Create(true);

   // Set maximum files
   m_maxFiles := 25;

   // Reset popup menu
   m_popupMenu := nil;

   // Reset
   Reset;
end;

// Destructor
//
destructor CLastOpenFiles.Destroy;
begin
	try
   	// Free the file container
       FreeAndNil(m_files);
	finally
   	inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset
//
procedure CLastOpenFiles.Reset;
begin
	// Clear the files
	m_files.Clear;

   // Reset internal add
   m_internalAdd := false;
end;

// Add new file
//
procedure CLastOpenFiles.Add(fileName: string);
var
   f: CLastOpenFile;
begin
	// Check if file exists
   if not FileExists(fileName) then
   	exit;

	// Remove the old file
   removeFile(fileName);

   // Add the new file to the begining of the list
	f := CLastOpenFile.Create(self);

   // Set the file name
   f.FileName := fileName;

   // Add to the container
   if m_internalAdd then
   	m_files.Add(f)
   else
		m_files.Insert(0,f);

	// Collect the garbage
   collectGarbage;

   // Build the popup menu
   buildPopupMenu;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Remove the file
//
procedure CLastOpenFiles.removeFile(fileName: string);
var
	idx: integer;
begin
	// Check if file exists
	if hasFile(fileName,idx) then
   	// Delete the file
   	m_files.Delete(idx);
end;

// Collect the garbage
//
procedure CLastOpenFiles.collectGarbage;
var
	i: integer;
   f: CLastOpenFile;
begin
	// Remove old files
   while (count > MaxFiles) do
   	m_files.Delete(count - 1);

   // Remove non existing files
   for i := count - 1 downto 0 do
   begin
   	// Get the file
       f := CLastOpenFile(m_files[i]);

       // Delete the non existing file
       if not f.Exists then
       	m_files.Delete(i);
   end;
end;

// Determine if the file exists already
//
function CLastOpenFiles.hasFile(fileName: string; var idx: integer): boolean;
var
	i: integer;
	ucFileName: string;
   f: CLastOpenFile;
begin
	// Init index
   idx := -1;

   // Get the uppercase file name
   ucFileName := UpperCase(fileName);

   // Iterate file container
   for i := 0 to count - 1 do
   begin
   	// Get the file
       f := CLastOpenFile(m_files[i]);

       // Compare file name
       if (UpperCase(f.FileName) = ucFileName) then
       begin
       	// Return index
       	idx := i;

           // Break
           break;
       end;
   end;

   // Determine success
   result := (idx <> -1);
end;

// Get the file count
//
function CLastOpenFiles.count: integer;
begin
	result := m_files.Count;
end;

// Build the popup menu
//
procedure CLastOpenFiles.buildPopupMenu;
var
	i: integer;
   f: CLastOpenFile;
   mnuItem: TMenuItem;
begin
	// No popup menu given
	if not Assigned(m_popupMenu) then
   	exit;

   // Clear the items
   m_popupMenu.Items.Clear;

   // Add the files
   for i := 0 to count - 1 do
   begin
      	// Get the file
       f := CLastOpenFile(m_files[i]);

       // Create new menu item
       mnuItem := TMenuItem.Create(m_popupMenu);

       // Set the caption
       mnuItem.Caption := f.FileName;

       // Set the click event
       mnuItem.OnClick := onClick;

       // Add to the menu
       m_popupMenu.Items.Add(mnuItem);
   end;
end;

// On click of menu item
//
procedure CLastOpenFiles.onClick(Sender: TObject);
var
	fName: string;
begin
	if Assigned(m_onPopupItemClickEvent) then
   begin
       // Get the file name
       fName := TMenuItem(sender).Caption;

   	// Fire event
   	m_onPopupItemClickEvent(self,fName);
   end;
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the files as string
//
function CLastOpenFiles.getAsString: string;
var
	list: CDelimitedStringList;
   i: integer;
begin
	// Create the delimited string list
	list := CDelimitedStringList.Create;
	try
		// Add the values to the list
		for i := 0 to count - 1 do
			list.Add(CLastOpenFile(m_files[i]).FileName);

		// Return the delimiter text
		result := list.DelimitedText;
	finally
		// Free the list
		FreeAndNil(list);
	end;
end;

// Set the files as string
//
procedure CLastOpenFiles.setAsString(val: string);
var
	list: CDelimitedStringList;
   i: integer;
begin
	// Reset the files
   Reset;

	// Create the delimited string list
	list := CDelimitedStringList.Create;
	try
		// Set the delimited value
		list.DelimitedText := val;

       // Set internal add
       m_internalAdd := true;

       // Add the files
		for i := 0 to list.Count - 1 do
			Add(list[i]);
	finally
       // Reset internal add
       m_internalAdd := false;

		// Free the list
		FreeAndNil(list);
	end;
end;
