//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CLanguageManager.
//
//	Description:  	This unit implements the language manager class
//
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CLanguageManager.Create;
begin
	inherited;

	// Reset the language file
	LanguageFile.Reset;

	// Reset the translation definition
	m_translationRegistered := false;

	// Reset translation mode
	m_translationMode := false;

   // Reset translating flag
   m_translating := false;

   // Setup current language
   m_currentLanguage := CONST_DEFAULT_LANGUAGE;

   // Load the translations
   Translations.Load;
end;

// Destructor
//
destructor CLanguageManager.Destroy;
begin
	try
		// Free the forms
		if Assigned(m_forms) then
			FreeAndNil(m_forms);

		// Free the controls
		if Assigned(m_controls) then
			FreeAndNil(m_controls);

		// Free the variables
		if Assigned(m_variables) then
			FreeAndNil(m_variables);

		// Free the translateable form
		if Assigned(m_translateableForms) then
			FreeAndNil(m_translateableForms);

		// Free the language file
		if Assigned(m_languageFile) then
			FreeAndNil(m_languageFile);

       // Free the translations
       if Assigned(m_translations) then
       	FreeAndNil(m_translations);
	finally
		inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Register the translation
//
procedure CLanguageManager.RegisterTranslation(definitionClass: CTranslateDefinitionBaseClass);
var
	definition: CTranslateDefinitionBase;
begin
	// Check if already registerered
	if m_translationRegistered or (definitionClass = nil) then
		exit;

	// Set the translation registered flag
	m_translationRegistered := true;

	// Create the definition class
	definition := definitionClass.Create(self);
	try
		// Register the forms
		definition.RegisterForms;

		// Register the variables
		definition.RegisterVariables;
	finally
		// Free the definition
		FreeAndNil(definition);
	end;
end;

// Start the translation
//
procedure CLanguageManager.StartTranslation;
var
	frmTranslate: TFrmTranslateForm;
begin
   // Validate
   if m_translating then
   	exit;

	// Set translating flag
   m_translating := true;

	// Create the translator form
	frmTranslate := TFrmTranslateForm.Create(nil);
	try
		// Execute the form
		frmTranslate.Execute(self);
	finally
		// Free the translator form
		FreeAndNil(frmTranslate);

		// Reset translating flag
   	m_translating := false;

       // Fire translation end event
       if Assigned(m_onEndTranslation) then
       	m_onEndTranslation(self);
	end;
end;

// Translate the given form
//
procedure CLanguageManager.TranslateForm(frm: TForm);
var
	translateForm: CTranslateForm;
	formControl: CTranslateFormControl;
	ctrl: CTranslateControl;
	i: integer;
begin
	// Load the form controls
	Controls.LoadFromForm(frm);

	// Check for controls
	if (Controls.Count = 0) then
		exit;

	// Get the form
	translateForm := Forms.FormByName[frm.Name];

	// Iterate controls
	for i := 0 to translateForm.Controls.Count - 1 do
	begin
		// Get the form control
		formControl := translateForm.Controls.Control[i];

  		// Get the control
		ctrl := Controls.ControlByName[formControl.ControlName];

       // Validate control
       if (ctrl <> nil) then
			// Update the caption
			ctrl.Caption := formControl.Caption;
	end;
end;

// Choose language
//
procedure CLanguageManager.ChooseLanguage;
var
	frmChooseTranslation: TFrmChooseTranslation;
begin
	// Create the choose translation form
	frmChooseTranslation := TFrmChooseTranslation.Create(nil);
	try
		// Execute the form
		frmChooseTranslation.Execute(self);
	finally
		// Free the choose translation form
		FreeAndNil(frmChooseTranslation);
	end;
end;

// Change the language
//
procedure CLanguageManager.ChangeLanguage(language: string);
var
	translation: CTranslation;
begin
   // Compare if language was changed
   if (CurrentLanguage = language) then
   	exit;

   // Select the deafult language
   if (language = CONST_DEFAULT_LANGUAGE) then
   begin
   	// Change to default
   	m_currentLanguage := CONST_DEFAULT_LANGUAGE;

       // Nothing more to do
       exit;
   end;

   // Get the translation
	translation := Translations.TranslationByName[language];

   // Load the translation
   if (translation <> nil) then
   begin
       // Load language file
       LanguageFile.Load(translation.FileName);

		// Store current language
		m_currentLanguage := language;

       // Fire the change language notification
       if Assigned(m_onChangeLanguage) and not m_translating then
       	m_onChangeLanguage(self,language);
   end;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the controls
//
function CLanguageManager.getControls: CTranslateControls;
begin
	// Create controls on demand
	if not Assigned(m_controls) then
		m_controls := CTranslateControls.Create(self);

	// Return instance
	result := m_controls;
end;

// Get the variables
//
function CLanguageManager.getVariables: CTranslateVariables;
begin
	// Create variables on demand
	if not Assigned(m_variables) then
		m_variables := CTranslateVariables.Create(self);

	// Return instance
	result := m_variables;
end;

// Get the forms
//
function CLanguageManager.getForms: CTranslateForms;
begin
	// Create forms on demand
	if not Assigned(m_forms) then
		m_forms := CTranslateForms.Create(self);

	// Return instance
	result := m_forms;
end;

// Get the translateable forms
//
function CLanguageManager.getTranslateableForms: CTranslateableForms;
begin
	// Create forms on demand
	if not Assigned(m_translateableForms) then
		m_translateableForms := CTranslateableForms.Create(self);

	// Return instance
	result := m_translateableForms;
end;

// Get the language file
//
function CLanguageManager.getLanguageFile: CLanguageFile;
begin
	// Create language file on demand
	if not Assigned(m_languageFile) then
		m_languageFile := CLanguageFile.Create(self);

	// Return instance
	result := m_languageFile;
end;

// Get the translations
//
function CLanguageManager.getTranslations: CTranslations;
begin
	// Create translations on demand
	if not Assigned(m_translations) then
		m_translations := CTranslations.Create(self);

	// Return instance
	result := m_translations;
end;

// Get the language directory
//
function CLanguageManager.getLanguageDirectory: string;
begin
	// Determine path
	result := PathIncludeTrailingBackslash(ExtractFilePath(ParamStr(0))) + CONST_LANGUAGEFOLDER_LANGUAGE;

	// Make sure directory exists
	if not DirectoryExists(result) then
		CreateDir(result);
end;

// Get the translation file name
//
function CLanguageManager.getTranslationFileName: string;
var
	filePath: string;
begin
	// Determine file path
	filePath := PathIncludeTrailingBackslash(ExtractFilePath(ParamStr(0))) + CONST_LANGUAGEFOLDER_TRANSLATION;

	// Make sure directory exists
	if not DirectoryExists(filePath) then
		CreateDir(filePath);

	// Build filename
	result := filePath + '\' + CONST_LANGUAGEFILE_TRANSLATION;
end;
