//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of TFrmTranslateForm.
//
//	Description:  	This unit implements the user interface to translate the
//					controls
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

unit UFrmTranslateForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, CLanguages, ComCtrls, StdCtrls;

type
	// The language translate form
	//
	TFrmTranslateForm = class(TForm)
		pnlTranslateFormBase: TPanel;
		cmdOk: TButton;
		cmdCancel: TButton;
		pnlValuesBase: TPanel;
		splSplitter: TSplitter;
		grpControls: TGroupBox;
		lvwControls: TListView;
		grpVariables: TGroupBox;
		lvwVariables: TListView;
		cmdResetVariables: TButton;
		cboTranslateableForms: TComboBox;
    	lblForm: TLabel;
		procedure lvwControlsEdited(Sender: TObject; Item: TListItem; var S: string);
		procedure lvwVariablesEdited(Sender: TObject; Item: TListItem; var S: string);
		procedure cmdResetVariablesClick(Sender: TObject);
		procedure cboTranslateableFormsChange(Sender: TObject);
    	procedure FormDestroy(Sender: TObject);
	// Private declarations
	private
		m_manager: CLanguageManager;
		m_translateForm: TForm;
	private
		procedure loadTranslateableForms;
		procedure loadControls;
		procedure loadVariables;
		procedure storeCurrentForm;
		procedure updateForm(formName: string);
	// Public declarations
	public
		function Execute(manager: CLanguageManager): boolean;
	end;

var
  FrmTranslateForm: TFrmTranslateForm;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
// Form creation and destruction
//------------------------------------------------------------------------------

// Form destroy notification
//
procedure TFrmTranslateForm.FormDestroy(Sender: TObject);
begin
	// Hide all translateable forms
	m_manager.TranslateableForms.HideAll;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Load the controls from manager
//
function TFrmTranslateForm.Execute(manager: CLanguageManager): boolean;
begin
	// Set manager reference
	m_manager := manager;

	// Load the default translation
   m_manager.LanguageFile.Load(theLanguageManager.TranslationFileName);

	// Load the translateable forms
	loadTranslateableForms;

	// Set the item index to first form
	cboTranslateableForms.ItemIndex := 0;

	// Force change event
	cboTranslateableFormsChange(nil);

	// Determine success
	result := (ShowModal = mrOk);

	// Determine success
	if result then
	begin
		// Store the current form
		storeCurrentForm;

		// Save to translation file
		m_manager.LanguageFile.Save(m_manager.TranslationFileName);
	end;
end;

//------------------------------------------------------------------------------
// Control events
//------------------------------------------------------------------------------

// Reset the variables
//
procedure TFrmTranslateForm.cmdResetVariablesClick(Sender: TObject);
begin
	// Undo changes on variables
	m_manager.Variables.UndoChanges;

	// Reload variables
   loadVariables;
end;


// On edited of lvwControls
//
procedure TFrmTranslateForm.lvwControlsEdited(Sender: TObject; Item: TListItem; var S: string);
var
	control: CTranslateControl;
begin
	if (item <> nil) and (item.Data <> nil) then
	begin
		// Get the control
		control := CTranslateControl(item.Data);

		// Update the caption
		control.Caption := s;
	end;
end;

// On edited of lvwVariables
//
procedure TFrmTranslateForm.lvwVariablesEdited(Sender: TObject; Item: TListItem; var S: string);
var
	variable: CTranslateVariable;
begin
	if (item <> nil) and (item.Data <> nil) then
	begin
		// Get the control
		variable := CTranslateVariable(item.Data);

		// Update the text
		variable.Text := s;
	end;
end;

// On change of cboTranslateableForms
//
procedure TFrmTranslateForm.cboTranslateableFormsChange(Sender: TObject);
begin
	// Store the current form
	storeCurrentForm;

	// Hide all translateable forms
	m_manager.TranslateableForms.HideAll;

	// Get the form
	m_translateForm := m_manager.TranslateableForms.Form[cboTranslateableForms.ItemIndex].FormInstance;

   // Translate the form
   m_manager.TranslateForm(m_translateForm);

	// Show the form
	m_translateForm.Show;

	// Load the form controls
	m_manager.Controls.LoadFromForm(m_translateForm);

	// Load controls
	loadControls;

	// Load variables
	loadVariables;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Load the translateable forms
//
procedure TFrmTranslateForm.loadTranslateableForms;
var
	i: integer;
	translateableForm: CTranslateableForm;
begin
	// Clear the translateable forms
	cboTranslateableForms.Clear;

	// Load all translateable forms
	for i := 0 to m_manager.TranslateableForms.Count - 1 do
	begin
		// Get the form
		translateableForm := m_manager.TranslateableForms.Form[i];

		// Add the form name
		cboTranslateableForms.Items.AddObject(translateableForm.FormName,translateableForm);
	end;
end;

// Load the controls
//
procedure TFrmTranslateForm.loadControls;
var
	control: CTranslateControl;
	item: TListItem;
	i: integer;
begin
	// Clear the items
	lvwControls.Items.Clear;

	// Add the controls to the list view
	for i := 0 to m_manager.Controls.Count - 1 do
	begin
		// Get the control
		control := m_manager.Controls[i];

		// Add control to list view
		item := lvwControls.Items.Add;

		// Setup item properties
		item.Caption := control.Caption;
		item.SubItems.Add(control.Name);
		item.Data := control;
	end;
end;

// Load the variables
//
procedure TFrmTranslateForm.loadVariables;
var
	variable: CTranslateVariable;
	item: TListItem;
	i: integer;
begin
	// Clear the items
	lvwVariables.Items.Clear;

	// Add the items to the list view
	for i := 0 to m_manager.Variables.Count - 1 do
	begin
		// Get the control
		variable := m_manager.Variables.Variable[i];

		// Add variable to list view
		item := lvwVariables.Items.Add;

		// Setup item properties
		item.Caption := variable.Text;
		item.SubItems.Add(variable.Name);
		item.Data := variable;
	end;
end;

// Store the current form
//
procedure TFrmTranslateForm.storeCurrentForm;
begin
	// Check if translate form is assigned
	if Assigned(m_translateForm) then
		updateForm(m_translateForm.Name);
end;

// Update the form
//
procedure TFrmTranslateForm.updateForm(formName: string);
var
	frm: CTranslateForm;
	formControl: CTranslateFormControl;
	ctrl: CTranslateControl;
	i: integer;
begin
	// Check for controls
	if (m_manager.Controls.Count = 0) then
		exit;

	// Get the form
	frm := m_manager.Forms.FormByName[formName];

	// Iterate controls
	for i := 0 to m_manager.Controls.Count - 1 do
	begin
		// Get the control
		ctrl := m_manager.Controls[i];

		// Get the form control
		formControl := frm.Controls.ControlByName[ctrl.Name];

		// Update the caption
		formControl.Caption := ctrl.Caption;
	end;
end;

end.
