//------------------------------------------------------------------------------
// Guitar Practice
//------------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		The guitar practice options form.
//
//	Description:   	This unit implements the options form of the
//                 guitar practice application
//
// Created:		FeMann
// Date:			15.03.2010
//
// (c) Copyright by FeMann.
// All rights reserved.
//
//

unit UFrmGuitarPracticeOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, CGPClasses, Buttons, 
  {$WARNINGS OFF} FileCtrl {$WARNINGS ON};

type
	// The guitar practice options form
	//
	TFrmGuitarPracticeOptions = class(TForm)
		pnlOptionsTop: TPanel;
		pnlOptionsBottom: TPanel;
		pgcOptions: TPageControl;
		tbsGeneral: TTabSheet;
		tbsEncoder: TTabSheet;
		cmdOk: TButton;
		grpEncoderOptions: TGroupBox;
		lblEncoder: TLabel;
		cmdSelectEncoder: TSpeedButton;
		lblEncoderCommandline: TLabel;
		txtEncoderCommandline: TEdit;
		cmdSelectEncoderDefaults: TSpeedButton;
		lblEncoderOutputpath: TLabel;
		txtEncoderOutputPath: TEdit;
		cmdSelectOutputPath: TSpeedButton;
		txtEncoder: TEdit;
		grpGeneralOptions: TGroupBox;
		lblKeepOpenFileHistory: TLabel;
		cboKeepFileHistory: TComboBox;
		lblKeepFileHistoryEntries: TLabel;
		procedure FormCreate(Sender: TObject);
		procedure cmdSelectEncoderClick(Sender: TObject);
		procedure cmdSelectOutputPathClick(Sender: TObject);
		procedure cmdSelectEncoderDefaultsClick(Sender: TObject);
		// Private declarations
		private
			m_manager: CGPManager;
		private
			procedure loadOptions;
			procedure saveOptions;
			procedure loadEncoderSettings;
			procedure saveEncoderSettings;
			procedure loadGeneralSettings;
			procedure saveGeneralSettings;
			procedure initDefaults;
			procedure initKeepFileHistoryCombo;
			function selectDir(root: string): string;
			procedure resolveEncoderDefaultCommandLine;
		// Public declarations
		public
			function Execute(manager: CGPManager): boolean;
	end;

var
	FrmGuitarPracticeOptions: TFrmGuitarPracticeOptions;

implementation

uses
	CGPconst, CLanguages;

{$R *.dfm}

//------------------------------------------------------------------------------
// Constructor and destructor
//------------------------------------------------------------------------------

// Form create notification
//
procedure TFrmGuitarPracticeOptions.FormCreate(Sender: TObject);
begin
	// Setup form caption
	Caption := theLanguageManager.Variables[GP_FORM_CAPTION_OPTIONS];

	// Setup page control
	pgcOptions.ActivePageIndex := 0;

	// Translate the form
	theLanguageManager.TranslateForm(self);
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Execute the options dialog
//
function TFrmGuitarPracticeOptions.Execute(manager: CGPManager): boolean;
begin
	// Set backward reference to manager
	m_manager := manager;

	// Init the defaults
	initDefaults;
	
	// Load the options
	loadOptions;
	
	// Show modal dialog
	ShowModal;

	// Determine success
	result := (ModalResult = mrOk);

	// Save options
	if result then
		saveOptions;
end;

//------------------------------------------------------------------------------
// Form methods
//------------------------------------------------------------------------------

// Select the encoder
//
procedure TFrmGuitarPracticeOptions.cmdSelectEncoderClick(Sender: TObject);
var
	dlg: TOpenDialog;
begin
	// Create the dialog
	dlg := TOpenDialog.Create(self);
	try
		// Setup properties
		dlg.DefaultExt := 'exe';
		dlg.Filter := 'Executables|*.exe';

		// Execute the dialog
		if dlg.Execute then
		begin
			// Set encoder
			txtEncoder.Text := dlg.FileName;

			// Resolve default command line
			resolveEncoderDefaultCommandLine;
       end;
	finally
		// Free the dialog
		FreeAndNil(dlg);
   end;
end;

// Select the output path
//
procedure TFrmGuitarPracticeOptions.cmdSelectOutputPathClick(Sender: TObject);
begin
	txtEncoderOutputPath.Text := selectDir(txtEncoderOutputPath.Text);
end;

// Resolve encoder command line
//
procedure TFrmGuitarPracticeOptions.cmdSelectEncoderDefaultsClick(Sender: TObject);
begin
	resolveEncoderDefaultCommandLine;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Load the options
//
procedure TFrmGuitarPracticeOptions.loadOptions;
begin
	// Load the general settings
	loadGeneralSettings;

	// Load the encoder settings
	loadEncoderSettings;
end;

// Save the options
//
procedure TFrmGuitarPracticeOptions.saveOptions;
begin
	// Save the general settings
	saveGeneralSettings;

	// Save the encoder settings
	saveEncoderSettings;
end;

// Load the general settings
//
procedure TFrmGuitarPracticeOptions.loadGeneralSettings;
begin
	// Load the keep file history combo box
	cboKeepFileHistory.ItemIndex := cboKeepFileHistory.Items.IndexOf(IntToStr(m_manager.Options.KeepFileHistory));
end;

// Save the general settings
//
procedure TFrmGuitarPracticeOptions.saveGeneralSettings;
begin
	// Save the keep file history combo box
	m_manager.Options.KeepFileHistory := StrToInt(cboKeepFileHistory.Text);
end;

// Load the encoder settings
//
procedure TFrmGuitarPracticeOptions.loadEncoderSettings;
begin
	// Load encoder
	if (m_manager.Options.Encoder <> '') and FileExists(m_manager.Options.Encoder) then
		txtEncoder.Text := m_manager.Options.Encoder;

	// Load encoder output directory
	if (m_manager.Options.EncoderOutputDirectory <> '') and DirectoryExists(m_manager.Options.EncoderOutputDirectory) then
		txtEncoderOutputPath.Text := m_manager.Options.EncoderOutputDirectory;

	// Load encoder command line
	txtEncoderCommandline.Text := m_manager.Options.EncoderCommandLine;	
end;

// Save the encoder settings
//
procedure TFrmGuitarPracticeOptions.saveEncoderSettings;
begin
	m_manager.Options.Encoder := txtEncoder.Text;
	m_manager.Options.EncoderOutputDirectory := txtEncoderOutputPath.Text; 
	m_manager.Options.EncoderCommandLine := txtEncoderCommandline.Text;
end;

// Init the defaults
//
procedure TFrmGuitarPracticeOptions.initDefaults;
begin
	// Init the keep file history combo box
	initKeepFileHistoryCombo;
end;

// Init keep file history combo box
//
procedure TFrmGuitarPracticeOptions.initKeepFileHistoryCombo;
var
	i: integer;
begin
	// Clear items
	cboKeepFileHistory.Clear;

	// Fill combo box with 5 to 50 entries (step by 5)
	for i := 1 to 10 do
		cboKeepFileHistory.Items.Add(IntToStr(i * 5));
end;

// Select the directory
//
function TFrmGuitarPracticeOptions.selectDir(root: string): string;
begin
	// Init result
	result := root;

	// Select the directory
	SelectDirectory(result, [sdAllowCreate, sdPerformCreate, sdPrompt], 0);
end;

// Resolve the encoder default command line
//
procedure TFrmGuitarPracticeOptions.resolveEncoderDefaultCommandLine;
var
	exeName: string;
	defaultCommandLine: string;
begin
	// Nothing more to do
	if (txtEncoder.Text = '') then
		exit;
	
	// Get the exe name
	exeName := ExtractFileName(txtEncoder.Text);

	// Validate exe name
	if (exeName <> '') then
	begin
		// Get the default command line
		defaultCommandLine := m_manager.Options.EncoderDefaultCommandLineByName[exeName];
		
		// Set default command line
		if (defaultCommandLine <> '') then
			txtEncoderCommandline.Text := defaultCommandLine;
	end;
end;

end.
